<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: DalMaintenance.dal.test.php 3635 2005-09-05 16:27:58Z andrew $
*/

require_once MAX_PATH . '/lib/core/ServiceLocator.php';
require_once MAX_PATH . '/lib/max/Dal/Maintenance.php';
require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/lib/max/Table/Priority.php';
require_once 'Date.php';
require_once 'DB/QueryTool.php';

/**
 * A class for testing the non-DB specific Maintenance Engine DAL class.
 *
 * @package    MaxDal
 * @subpackage TestSuite
 * @author     Andrew Hill <andrew@m3.net>
 * @author     Demian Turner <demian@m3.net>
 * @author     James Floyd <james@m3.net>
 */
class Dal_TestOfDalMaintenance extends UnitTestCase
{
    
    /**
     * The constructor method.
     */
    function Dal_TestOfDalMaintenance()
    {        
        $this->UnitTestCase();
    }
    
    /**
     * Method to test the singleton method.
     *
     * Requirements:
     * Test 1: Test that only one instance of the class is created.
     */
    function testSingleton()
    {
        // Mock the MAX_DB class used in the constructor method
        Mock::generate('MAX_DB');
        $dbh = &new MockMAX_DB($this);
        
        // Mock the DB_QueryTool class used in the constructor method
        Mock::generate('DB_QueryTool');
        $queryBuilder = &new MockDB_QueryTool($this);
        
        // Partially mock the MAX_Dal_Maintenance class
        Mock::generatePartial('MAX_Dal_Maintenance',
                              'PartialMockMAX_Dal_Maintenance',
                              array('_createMaxDb', '_getQueryTool'));
        $oMaxDalMaintenance = &new PartialMockMAX_Dal_Maintenance($this);
        $oMaxDalMaintenance->setReturnReference('_createMaxDb', $dbh);
        $oMaxDalMaintenance->setReturnReference('_getQueryTool', $queryBuilder);
        
        // Test 1
        $first  = &$oMaxDalMaintenance->singleton();
        $second = &$oMaxDalMaintenance->singleton();
        $this->assertIdentical($first, $second);
    }
    
    /**
     * Method to test the setMaintenancePriorityLastRunInfo method.
     *
     * Requirements:
     * Test 1: Test with no data in the database, ensure data is correctly stored.
     * Test 2: Test with previous test data in the database, ensure data is correctly stored.
     */
    function testSetMaintenancePriorityLastRunInfo()
    {
        TestEnv::startTransaction();
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        
        // Test 1
        $oStartDate = new Date('2005-06-21 15:00:01');
        $oEndDate   = new Date('2005-06-21 15:01:01');
        $oUpdatedTo = new Date('2005-06-21 15:59:59');
        $oMaxDalMaintenance->setMaintenancePriorityLastRunInfo($oStartDate, $oEndDate, $oUpdatedTo);
        $query = "
            SELECT
                start_run,
                end_run,
                operation_interval,
                duration,
                updated_to
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_priority']}
            WHERE
                log_maintenance_priority_id = 1";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['start_run'], '2005-06-21 15:00:01');
        $this->assertEqual($row['end_run'], '2005-06-21 15:01:01');
        $this->assertEqual($row['operation_interval'], $conf['maintenance']['operationInterval']);
        $this->assertEqual($row['duration'], 60);
        $this->assertEqual($row['updated_to'], '2005-06-21 15:59:59');
        
        // Test 2
        $oStartDate = new Date('2005-06-21 16:00:01');
        $oEndDate   = new Date('2005-06-21 16:01:06');
        $oUpdatedTo = new Date('2005-06-21 16:59:59');
        $oMaxDalMaintenance->setMaintenancePriorityLastRunInfo($oStartDate, $oEndDate, $oUpdatedTo);
        $query = "
            SELECT
                start_run,
                end_run,
                operation_interval,
                duration,
                updated_to
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_priority']}
            WHERE
                log_maintenance_priority_id = 1";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['start_run'], '2005-06-21 15:00:01');
        $this->assertEqual($row['end_run'], '2005-06-21 15:01:01');
        $this->assertEqual($row['operation_interval'], $conf['maintenance']['operationInterval']);
        $this->assertEqual($row['duration'], 60);
        $this->assertEqual($row['updated_to'], '2005-06-21 15:59:59');
        $query = "
            SELECT
                start_run,
                end_run,
                operation_interval,
                duration,
                updated_to
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_priority']}
            WHERE
                log_maintenance_priority_id = 2";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['start_run'], '2005-06-21 16:00:01');
        $this->assertEqual($row['end_run'], '2005-06-21 16:01:06');
        $this->assertEqual($row['operation_interval'], $conf['maintenance']['operationInterval']);
        $this->assertEqual($row['duration'], 65);
        $this->assertEqual($row['updated_to'], '2005-06-21 16:59:59');
        
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Method to test the setMaintenanceStatisticsLastRunInfo method.
     *
     * Requirements:
     * Test 1: Test with no data in the database, ensure data is correctly stored.
     * Test 2: Test with previous test data in the database, ensure data is correctly stored.
     * Test 3: Test with previous test data in the database, for type tracker_ad_type
     */
    function testSetMaintenanceStatisticsLastRunInfo()
    {
        TestEnv::startTransaction();
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        
        // Test 1
        $oStartDate = new Date('2005-06-21 15:00:01');
        $oEndDate   = new Date('2005-06-21 15:01:01');
        $oUpdatedTo = new Date('2005-06-21 15:59:59');
        $runTypeField = 'adserver_run_type';
        $type = 2;
        $oMaxDalMaintenance->setMaintenanceStatisticsLastRunInfo($oStartDate, $oEndDate, $type, $oUpdatedTo, $runTypeField);
        $query = "
            SELECT
                start_run,
                end_run,
                duration,
                {$runTypeField},
                updated_to
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']}
            WHERE
                log_maintenance_statistics_id = 1";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['start_run'], '2005-06-21 15:00:01');
        $this->assertEqual($row['end_run'], '2005-06-21 15:01:01');
        $this->assertEqual($row[$runTypeField], $type);
        $this->assertEqual($row['duration'], 60);
        $this->assertEqual($row['updated_to'], '2005-06-21 15:59:59');
        
        // Test 2
        $oStartDate = new Date('2005-06-21 16:00:01');
        $oEndDate   = new Date('2005-06-21 16:01:06');
        $oUpdatedTo = new Date('2005-06-21 16:59:59');
        $runTypeField = 'adserver_run_type';
        $type = 2;
        $oMaxDalMaintenance->setMaintenanceStatisticsLastRunInfo($oStartDate, $oEndDate, $type, $oUpdatedTo, $runTypeField);
        $query = "
            SELECT
                start_run,
                end_run,
                duration,
                {$runTypeField},
                updated_to
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']}
            WHERE
                log_maintenance_statistics_id = 1";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['start_run'], '2005-06-21 15:00:01');
        $this->assertEqual($row['end_run'], '2005-06-21 15:01:01');
        $this->assertEqual($row[$runTypeField], $type);
        $this->assertEqual($row['duration'], 60);
        $this->assertEqual($row['updated_to'], '2005-06-21 15:59:59');
        $query = "
            SELECT
                start_run,
                end_run,
                duration,
                {$runTypeField},
                updated_to
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']}
            WHERE
                log_maintenance_statistics_id = 2";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['start_run'], '2005-06-21 16:00:01');
        $this->assertEqual($row['end_run'], '2005-06-21 16:01:06');
        $this->assertEqual($row[$runTypeField], $type);
        $this->assertEqual($row['duration'], 65);
        $this->assertEqual($row['updated_to'], '2005-06-21 16:59:59');
        
        // Test 3:
        $oStartDate = new Date('2005-06-21 16:00:01');
        $oEndDate   = new Date('2005-06-21 16:02:07');
        $oUpdatedTo = new Date('2005-06-21 16:59:59');
        $runTypeField = 'tracker_run_type';
        $type = 2;
        $oMaxDalMaintenance->setMaintenanceStatisticsLastRunInfo($oStartDate, $oEndDate, $type, $oUpdatedTo, $runTypeField);
        $query = "
            SELECT
                start_run,
                end_run,
                duration,
                {$runTypeField},
                updated_to
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']}
            WHERE
                log_maintenance_statistics_id = 3";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['start_run'], '2005-06-21 16:00:01');
        $this->assertEqual($row['end_run'], '2005-06-21 16:02:07');
        $this->assertEqual($row[$runTypeField], $type);
        $this->assertEqual($row['duration'], 126);
        $this->assertEqual($row['updated_to'], '2005-06-21 16:59:59');
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Method to test the testSetMaintenanceStatisticesRunReport method.
     *
     * Requirements:
     * Test 1: Test two writes to reports.
     */
    function testSetMaintenanceStatisticesRunReport()
    {
        TestEnv::startTransaction();
        require_once MAX_PATH . '/lib/max/other/lib-userlog.inc.php';
        
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        
        // Test 1
        $report = 'Maintenance run has finished :: Maintenance will run again at XYZ.';
        $oMaxDalMaintenance->setMaintenanceStatisticesRunReport($report);
        
        $query = "
            SELECT
                timestamp,
                usertype,
                userid,
                action,
                object,
                details
            FROM
                {$conf['table']['prefix']}{$conf['table']['userlog']}
            WHERE
                userlogid = 1";
        
        $row = $dbh->getRow($query);
        $this->assertEqual($row['usertype'], phpAds_userMaintenance);
        $this->assertEqual($row['userid'], '0');
        $this->assertEqual($row['action'], phpAds_actionBatchStatistics);
        $this->assertEqual($row['object'], '0');
        $this->assertEqual($row['details'], $report);
        
        $report = '2nd Maintenance run has finished :: Maintenance will run again at XYZ.';
        $oMaxDalMaintenance->setMaintenanceStatisticesRunReport($report);
        
        $query = "
            SELECT
                timestamp,
                usertype,
                userid,
                action,
                object,
                details
            FROM
                {$conf['table']['prefix']}{$conf['table']['userlog']}
            WHERE
                userlogid = 2";
        
           $row = $dbh->getRow($query);
           $this->assertEqual($row['usertype'], phpAds_userMaintenance);
           $this->assertEqual($row['userid'], '0');
           $this->assertEqual($row['action'], phpAds_actionBatchStatistics);
           $this->assertEqual($row['object'], '0');
           $this->assertEqual($row['details'], $report);
           TestEnv::rollbackTransaction();
    }
    
    /**
     * Method to test the getMaintenancePriorityLastRunInfo method.
     *
     * Requirements:
     * Test 1: Test correct results are returned with no data.
     * Test 2: Test correct results are returned with single data entry.
     * Test 3: Test correct results are returned with multiple data entries.
     */
    function testGetMaintenancePriorityLastRunInfo()
    {
        TestEnv::startTransaction();
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        
        // Test 1
        $result = $oMaxDalMaintenance->getMaintenancePriorityLastRunInfo();
        $this->assertFalse($result);
        
        // Test 2
        $oStartDate = new Date('2005-06-21 15:00:01');
        $oEndDate   = new Date('2005-06-21 15:01:01');
        $oUpdatedTo = new Date('2005-06-21 15:59:59');
        $oMaxDalMaintenance->setMaintenancePriorityLastRunInfo($oStartDate, $oEndDate, $oUpdatedTo);
        $result = $oMaxDalMaintenance->getMaintenancePriorityLastRunInfo();
        $this->assertTrue(is_array($result));
        $this->assertEqual($result['updated_to'], '2005-06-21 15:59:59');
        $this->assertEqual($result['operation_interval'], $conf['maintenance']['operationInterval']);
        
        // Test 3
        $oStartDate = new Date('2005-06-21 14:00:01');
        $oEndDate   = new Date('2005-06-21 14:01:01');
        $oUpdatedTo = new Date('2005-06-21 14:59:59');
        $oMaxDalMaintenance->setMaintenancePriorityLastRunInfo($oStartDate, $oEndDate, $oUpdatedTo);
        $result = $oMaxDalMaintenance->getMaintenancePriorityLastRunInfo();
        $this->assertTrue(is_array($result));
        $this->assertEqual($result['updated_to'], '2005-06-21 15:59:59');
        $this->assertEqual($result['operation_interval'], $conf['maintenance']['operationInterval']);
        $oStartDate = new Date('2005-06-21 16:00:01');
        $oEndDate   = new Date('2005-06-21 16:01:01');
        $oUpdatedTo = new Date('2005-06-21 16:59:59');
        $oMaxDalMaintenance->setMaintenancePriorityLastRunInfo($oStartDate, $oEndDate, $oUpdatedTo);
        $result = $oMaxDalMaintenance->getMaintenancePriorityLastRunInfo();
        $this->assertTrue(is_array($result));
        $this->assertEqual($result['updated_to'], '2005-06-21 16:59:59');
        $this->assertEqual($result['operation_interval'], $conf['maintenance']['operationInterval']);
        
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Method to test the getMaintenanceStatisticsLastRunInfo method.
     *
     * @TODO Not implemented.
     */
    function testGetMaintenanceStatisticsLastRunInfo() {}
    
    /**
     * A method to test the getPlacements method.
     */
    function testGetPlacements()
    {
        $da = MAX_Dal_Maintenance::singleton();
        TestEnv::startTransaction();
        $this->_generateStatsOne();
        $ret = $da->getPlacements();
                
        $this->assertTrue(is_array($ret));
        $this->assertTrue(count($ret) == 5);
        $campaign = $ret[0];
        $this->assertTrue(array_key_exists('campaignid', $campaign));
        $this->assertTrue(array_key_exists('views', $campaign));
        $this->assertTrue(array_key_exists('clicks', $campaign));
        $this->assertTrue(array_key_exists('conversions', $campaign));
        $this->assertTrue(array_key_exists('expire', $campaign));
        $this->assertTrue(array_key_exists('target_impression', $campaign));
        $this->assertTrue(array_key_exists('target_click', $campaign));
        $this->assertTrue(array_key_exists('target_conversion', $campaign));
        $this->assertTrue(array_key_exists('priority', $campaign));
        TestEnv::rollbackTransaction();
    }
    
    /**
     * A method to test the getPlacementData method.
     */
    function testGetPlacementData()
    {
        $da = MAX_Dal_Maintenance::singleton();
        TestEnv::startTransaction();
        $this->_generateStatsOne();
        $ret = $da->getPlacementData(1);
        $this->assertTrue(is_array($ret));
        $this->assertTrue(count($ret) == 1);
        $campaign = $ret[0];
        $this->assertTrue(array_key_exists('advertiser_id', $campaign));
        $this->assertTrue(array_key_exists('placement_id', $campaign));
        $this->assertTrue(array_key_exists('name', $campaign));
        $this->assertTrue(array_key_exists('active', $campaign));
        $this->assertTrue(array_key_exists('num_children', $campaign));
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Method to test the getPlacementDeliveryToDate method.
     *
     * Requirements:
     * Test 1: Test correct results are returned with no data.
     * Test 2: Test correct results are returned with single data entry.
     * Test 3: Test correct results are returned with multiple data entries.
     *
     * @TODO Incomplete test!
     */
    function testGetPlacementDeliveryToDate()
    {
        TestEnv::startTransaction();
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        
        // Test 1
        $result = $oMaxDalMaintenance->getPlacementDeliveryToDate(1);
        $this->assertTrue(is_array($result));
        $this->assertEqual(count($result), 0);
        
        // Test 2
        $today = '2005-06-24';
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    activate,
                    expire,
                    priority,
                    active,
                    views,
                    clicks,
                    conversions
                )
            VALUES
                (
                    1,
                    '2005-06-23',
                    '2005-06-25',
                    1,
                    't',
                    100,
                    200,
                    300
                )";
        $result = $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    campaignid,
                    bannerid
                )
            VALUES
                (
                    1,
                    1
                )";
        $result = $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad']}
                (
                    day,
                    hour,
                    ad_id,
                    requests,
                    impressions,
                    clicks,
                    conversions
                )
            VALUES
                (
                    '2005-06-24',
                    10,
                    1,
                    500,
                    475,
                    25,
                    5
                ),
                (
                    '2005-06-24',
                    11,
                    1,
                    500,
                    475,
                    25,
                    5
                )";
        $result = $dbh->query($query);
        $result = $oMaxDalMaintenance->getPlacementDeliveryToDate(1);
        $this->assertTrue(is_array($result));
        $this->assertEqual(count($result), 1);
        $this->assertEqual($result[0]['placement_id'], 1);
        $this->assertEqual($result[0]['sum_requests'], 1000);
        $this->assertEqual($result[0]['sum_views'], 950);
        $this->assertEqual($result[0]['sum_clicks'], 50);
        $this->assertEqual($result[0]['sum_conversions'], 10);
        /*
        
        // Test 3
        $oStartDate = new Date('2005-06-21 14:00:01');
        $oEndDate   = new Date('2005-06-21 14:01:01');
        $oUpdatedTo = new Date('2005-06-21 14:59:59');
        $oMaxDalMaintenance->setMaintenancePriorityLastRunInfo($oStartDate, $oEndDate, $oUpdatedTo);
        $result = $oMaxDalMaintenance->getMaintenancePriorityLastRunInfo();
        $this->assertTrue(is_array($result));
        $this->assertEqual($result['updated_to'], '2005-06-21 15:59:59');
        $this->assertEqual($result['operation_interval'], $conf['maintenance']['operationInterval']);
        $oStartDate = new Date('2005-06-21 16:00:01');
        $oEndDate   = new Date('2005-06-21 16:01:01');
        $oUpdatedTo = new Date('2005-06-21 16:59:59');
        $oMaxDalMaintenance->setMaintenancePriorityLastRunInfo($oStartDate, $oEndDate, $oUpdatedTo);
        $result = $oMaxDalMaintenance->getMaintenancePriorityLastRunInfo();
        $this->assertTrue(is_array($result));
        $this->assertEqual($result['updated_to'], '2005-06-21 16:59:59');
        $this->assertEqual($result['operation_interval'], $conf['maintenance']['operationInterval']);
        */
        
        TestEnv::rollbackTransaction();
    }
    
    /**
     * A method to test the getPlacementDeliveryToday method.
     *
     * @TODO Not implemented.
     */
    function testGetPlacementDeliveryToday()
    {
    }
    
    /**
     * A method to test the getPlacementStats method.
     */
    function testGetPlacementStats()
    {
        $da = MAX_Dal_Maintenance::singleton();
        TestEnv::startTransaction();
        $this->_generateStatsOne();
        $ret = $da->getPlacementStats(1);
        
        $this->assertTrue(is_array($ret));
        $this->assertTrue(count($ret) == 9);
        
        $this->assertTrue(array_key_exists('advertiser_id', $ret));
        $this->assertTrue(array_key_exists('placement_id', $ret));
        $this->assertTrue(array_key_exists('name', $ret));
        $this->assertTrue(array_key_exists('active', $ret));
        $this->assertTrue(array_key_exists('num_children', $ret));
        $this->assertTrue(array_key_exists('sum_requests', $ret));
        $this->assertTrue(array_key_exists('sum_views', $ret));
        $this->assertTrue(array_key_exists('sum_clicks', $ret));
        $this->assertTrue(array_key_exists('sum_conversions', $ret));
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Method to test the getAllZonesWithAllocInv method.
     *
     * Requirements:
     * Test 1: Test with no data, and ensure no data returned.
     * Test 2: Test with sample data, and ensure the correct data is returned.
     */
    function testGetAllZonesWithAllocInv()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        
        // Create the required temporary table for the tests
        $oTable = &MAX_Table_Priority::singleton();
        $oTable->createTable('tmp_ad_zone_impression');
        
        // Test 1
        $result = &$oMaxDalMaintenance->getAllZonesWithAllocInv();
        $this->assertEqual(count($result), 0);
        
        // Test 2
        $query = "
            INSERT INTO
                tmp_ad_zone_impression
                (
                    ad_id,
                    zone_id,
                    impressions
                )
            VALUES
                (
                    1,
                    1,
                    10
                ),
                (
                    1,
                    2,
                    10
                ),
                (
                    2,
                    2,
                    10
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllZonesWithAllocInv();
        $this->assertEqual(count($result), 3);
        $this->assertEqual($result[0]['zone_id'], 1);
        $this->assertEqual($result[0]['ad_id'], 1);
        $this->assertEqual($result[1]['zone_id'], 2);
        $this->assertEqual($result[1]['ad_id'], 1);
        $this->assertEqual($result[2]['zone_id'], 2);
        $this->assertEqual($result[2]['ad_id'], 2);
        TestEnv::restoreEnv();
    }
    
    /**
     * Method to test the getAllZonesImpInv method.
     *
     * Requirements:
     * Test 1: Test with no Date registered in the service locator, ensure false returned.
     * Test 2: Test with a Date registered in the service locator, no data in the database,
     *         and ensure no data is returned.
     * Test 3: Test with data NOT in the current OI, and ensure no data is returned.
     * Test 4: Test with data both in, and not in, the current OI, and ensure the correct
     *         data is returned.
     */
    function testGetAllZonesImpInv()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        
        // Test 1
        $oServiceLocator = &ServiceLocator::instance();
        $oServiceLocator->remove('Date');
        $result = &$oMaxDalMaintenance->getAllZonesImpInv();
        $this->assertFalse($result);
        
        // Test 2
        $oDate = &new Date();
        $oServiceLocator->register('now', $oDate);
        $result = &$oMaxDalMaintenance->getAllZonesImpInv();
        $this->assertEqual(count($result), 0);
        
        // Test 3
        TestEnv::startTransaction();
        $oDate = &$oServiceLocator->get('now');
        $oNewDate = &new Date();
        $oNewDate->copy($oDate);
        $oNewDate->subtractSeconds(($conf['maintenance']['operationInterval'] * 60) + 1);
        $oOperationInterval = &OperationInterval::singleton();
        $previousOptIntID = $oOperationInterval->getIntervalID($oNewDate);
        $aDates = $oOperationInterval->convertDateToOperationIntervalStartAndEndDates($oNewDate);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['data_summary_zone_impression_history']}
                (
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    zone_id,
                    forecast_impressions
                )
            VALUES
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOptIntID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    42
                )";
        $dbh->query($query);
        $oNewDate = &new Date();
        $oNewDate->copy($aDates['start']);
        $oNewDate->subtractSeconds(1);
        $previousPreviousOpIntID = $oOperationInterval->getIntervalID($oNewDate);
        $aDates = $oOperationInterval->convertDateToOperationIntervalStartAndEndDates($oNewDate);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['data_summary_zone_impression_history']}
                (
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    zone_id,
                    actual_impressions
                )
            VALUES
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousPreviousOpIntID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    37
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllZonesImpInv();
        $this->assertEqual(count($result), 0);
        TestEnv::rollbackTransaction();
        
        // Test 4
        TestEnv::startTransaction();
        $oDate = &$oServiceLocator->get('now');
        $currentOpIntID = $oOperationInterval->getIntervalID($oDate);
        $aDates = $oOperationInterval->convertDateToOperationIntervalStartAndEndDates($oDate);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['data_summary_zone_impression_history']}
                (
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    zone_id,
                    forecast_impressions,
                    actual_impressions
                )
            VALUES
                (
                    {$conf['maintenance']['operationInterval']},
                    $currentOpIntID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    42,
                    NULL
                ),
                (
                    {$conf['maintenance']['operationInterval']},
                    $currentOpIntID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    2,
                    1,
                    2
                )";
        $dbh->query($query);
        $oNewDate = &new Date();
        $oNewDate->copy($oDate);
        $oNewDate->subtractSeconds(($conf['maintenance']['operationInterval'] * 60) + 1);
        $previousOptIntID = $oOperationInterval->getIntervalID($oNewDate);
        $aDates = $oOperationInterval->convertDateToOperationIntervalStartAndEndDates($oNewDate);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['data_summary_zone_impression_history']}
                (
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    zone_id,
                    forecast_impressions,
                    actual_impressions
                )
            VALUES
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOptIntID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    37,
                    11
                ),
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOptIntID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    2,
                    3,
                    4
                ),
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOptIntID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    3,
                    5,
                    6
                )";
        $dbh->query($query);
        $oNewDate = &new Date();
        $oNewDate->copy($aDates['start']);
        $oNewDate->subtractSeconds(1);
        $previousPreviousOpIntID = $oOperationInterval->getIntervalID($oNewDate);
        $aDates = $oOperationInterval->convertDateToOperationIntervalStartAndEndDates($oNewDate);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['data_summary_zone_impression_history']}
                (
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    zone_id,
                    forecast_impressions,
                    actual_impressions
                )
            VALUES
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousPreviousOpIntID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    10,
                    9
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllZonesImpInv();
        $this->assertEqual(count($result), 2);
        $this->assertEqual($result[0]['zone_id'], 1);
        $this->assertEqual($result[0]['forecast_impressions'], 42);
        $this->assertEqual($result[0]['actual_impressions'], 11);
        $this->assertEqual($result[1]['zone_id'], 2);
        $this->assertEqual($result[1]['forecast_impressions'], 1);
        $this->assertEqual($result[1]['actual_impressions'], 4);
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Method to test the getAllDeliveryLimitedAds method.
     *
     * Requirements:
     * Test 1: Test with no data, and ensure no data returned.
     * Test 2: Test with an active ad that is not filtered, and ensure no data returned.
     * Test 3: Test with an active ad that is filtered (4 ways), and ensure the ad is returned.
     * Test 4: Test with an inactive ad that is not filtered, and ensure no data returned.
     * Test 5: Test with an inactive ad that is filtered (4 ways), and ensure no data is returned.
     * Test 6: Test with a combination of active/inactive ads that are filtered/not filtered,
     *         and ensure the correct data is returned.
     * Test 7: Repeat of Test 6, but with an inactive parent campaign, ensure no data returned.
     */
    function testGetAllDeliveryLimitedAds()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        
        // Test 1
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 0);
        
        // Test 2
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    't',
                    0,
                    0,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 0);
        TestEnv::rollbackTransaction();
        
        // Test 3
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    't',
                    1,
                    0,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 1);
        TestEnv::rollbackTransaction();
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    't',
                    0,
                    1,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 1);
        TestEnv::rollbackTransaction();
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    't',
                    0,
                    0,
                    1,
                    'true'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 1);
        TestEnv::rollbackTransaction();
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    't',
                    0,
                    0,
                    0,
                    'phpAds_aclCheckSource(\'foo\', \'==\', \'bar\')'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 1);
        TestEnv::rollbackTransaction();
        
        // Test 4
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    'f',
                    0,
                    0,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 0);
        TestEnv::rollbackTransaction();
        
        // Test 5
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    'f',
                    1,
                    0,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 0);
        TestEnv::rollbackTransaction();
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    'f',
                    0,
                    1,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 0);
        TestEnv::rollbackTransaction();
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    'f',
                    0,
                    0,
                    1,
                    'true'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 0);
        TestEnv::rollbackTransaction();
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    'f',
                    0,
                    0,
                    0,
                    'phpAds_aclCheckSource(\'foo\', \'==\', \'bar\')'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 0);
        TestEnv::rollbackTransaction();
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    't',
                    0,
                    0,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 0);
        TestEnv::rollbackTransaction();
        
        // Test 6
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    't'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    't',
                    1,
                    0,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    2,
                    1,
                    't',
                    0,
                    1,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    3,
                    1,
                    't',
                    0,
                    0,
                    1,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    4,
                    1,
                    't',
                    0,
                    0,
                    0,
                    'phpAds_aclCheckSource(\'foo\', \'==\', \'bar\')'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    5,
                    1,
                    'f',
                    1,
                    0,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    6,
                    1,
                    'f',
                    0,
                    1,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    7,
                    1,
                    'f',
                    0,
                    0,
                    1,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    8,
                    1,
                    'f',
                    0,
                    0,
                    0,
                    'phpAds_aclCheckSource(\'foo\', \'==\', \'bar\')'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 4);
        $this->assertEqual($result[0]['bannerid'], 1);
        $this->assertEqual($result[1]['bannerid'], 2);
        $this->assertEqual($result[2]['bannerid'], 3);
        $this->assertEqual($result[3]['bannerid'], 4);
        TestEnv::rollbackTransaction();
        
        // Test 7
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                (
                    campaignid,
                    active
                )
            VALUES
                (
                    1,
                    'f'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    1,
                    1,
                    't',
                    1,
                    0,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    2,
                    1,
                    't',
                    0,
                    1,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    3,
                    1,
                    't',
                    0,
                    0,
                    1,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    4,
                    1,
                    't',
                    0,
                    0,
                    0,
                    'phpAds_aclCheckSource(\'foo\', \'==\', \'bar\')'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    5,
                    1,
                    'f',
                    1,
                    0,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    6,
                    1,
                    'f',
                    0,
                    1,
                    0,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    7,
                    1,
                    'f',
                    0,
                    0,
                    1,
                    'true'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['banners']}
                (
                    bannerid,
                    campaignid,
                    active,
                    block,
                    capping,
                    session_capping,
                    compiledlimitation
                )
            VALUES
                (
                    8,
                    1,
                    'f',
                    0,
                    0,
                    0,
                    'phpAds_aclCheckSource(\'foo\', \'==\', \'bar\')'
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getAllDeliveryLimitedAds();
        $this->assertEqual(count($result), 0);
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Method to test the getLastOpIntAdDeliveryInfo method.
     *
     * Requirements:
     * Test 1: Test with no Date registered in the service locator, ensure false returned.
     * Test 2: Test with a Date registered in the service locator, no data in the database,
     *         and ensure no data is returned.
     * Test 3: Test with data NOT in the previous OI, and ensure no data is returned.
     * Test 4: Test with data in the previous OI, and ensure correct data is returned.
     */
    function testGetLastOpIntAdDeliveryInfo()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oOperationInterval = &OperationInterval::singleton();
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        
        // Test 1
        $oServiceLocator = &ServiceLocator::instance();
        $oServiceLocator->remove('Date');
        $result = &$oMaxDalMaintenance->getLastOperationIntervalAdDeliveryInfo();
        $this->assertFalse($result);
        
        // Test 2
        $oDate = &new Date();
        $oServiceLocator->register('now', $oDate);
        $result = &$oMaxDalMaintenance->getLastOperationIntervalAdDeliveryInfo();
        $this->assertEqual(count($result), 0);
        
        // Test 3
        $operationIntervalID = $oOperationInterval->getIntervalID($oDate);
        $aDates = $oOperationInterval->convertDateToOperationIntervalStartAndEndDates($oDate);
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['data_summary_ad_zone_assoc']}
                (
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    ad_id,
                    zone_id,
                    impressions_requested,
                    priority_factor,
                    past_zone_traffic_fraction
                )
            VALUES
                (
                    {$conf['maintenance']['operationInterval']},
                    $operationIntervalID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    1,
                    1,
                    0.5,
                    0.99
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getLastOperationIntervalAdDeliveryInfo();
        $this->assertEqual(count($result), 0);
        TestEnv::rollbackTransaction();
        
        // Test 3
        $operationIntervalID = $oOperationInterval->getIntervalID($oDate);
        $previousOperationIntervalID = $oOperationInterval->getPreviousIntervalID($operationIntervalID);
        $aDates = $oOperationInterval->convertDateToPreviousOperationIntervalStartAndEndDates($oDate);
        TestEnv::startTransaction();
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['data_summary_ad_zone_assoc']}
                (
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    ad_id,
                    zone_id,
                    impressions_requested,
                    priority_factor,
                    past_zone_traffic_fraction,
                    created,
                    expired
                )
            VALUES
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOperationIntervalID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    1,
                    1,
                    0.5,
                    0.99,
                    '" . $oDate->format('%Y-%m-%d %H:%M:%S') . "',
                    NULL
                ),
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOperationIntervalID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    2,
                    1,
                    1,
                    0.5,
                    0.88,
                    '" . $oDate->format('%Y-%m-%d %H:%M:%S') . "',
                    NULL
                ),
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOperationIntervalID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    2,
                    1,
                    0.7,
                    0.77,
                    '" . $oDate->format('%Y-%m-%d %H:%M:%S') . "',
                    NULL
                ),
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOperationIntervalID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    2,
                    2,
                    1,
                    0.3,
                    0.66,
                    '" . $oDate->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $oDate->format('%Y-%m-%d %H:%M:%S') . "'
                )";
        $dbh->query($query);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad']}
                (
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    ad_id,
                    zone_id,
                    impressions
                )
            VALUES
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOperationIntervalID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    1,
                    1
                ),
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOperationIntervalID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    2,
                    1,
                    2
                ),
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOperationIntervalID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    2,
                    3
                ),
                (
                    {$conf['maintenance']['operationInterval']},
                    $previousOperationIntervalID,
                    '" . $aDates['start']->format('%Y-%m-%d %H:%M:%S') . "',
                    '" . $aDates['end']->format('%Y-%m-%d %H:%M:%S') . "',
                    1,
                    2,
                    3
                )";
        $dbh->query($query);
        $result = &$oMaxDalMaintenance->getLastOperationIntervalAdDeliveryInfo();
        $this->assertEqual(count($result), 3);
        $this->assertEqual($result[0]['ad_id'], 1);
        $this->assertEqual($result[0]['zone_id'], 1);
        $this->assertEqual($result[0]['impressions_requested'], 1);
        $this->assertEqual($result[0]['priority_factor'], 0.5);
        $this->assertEqual($result[0]['past_zone_traffic_fraction'], 0.99);
        $this->assertEqual($result[0]['impressions'], 1);
        $this->assertEqual($result[1]['ad_id'], 1);
        $this->assertEqual($result[1]['zone_id'], 2);
        $this->assertEqual($result[1]['impressions_requested'], 1);
        $this->assertEqual($result[1]['priority_factor'], 0.7);
        $this->assertEqual($result[1]['past_zone_traffic_fraction'], 0.77);
        $this->assertEqual($result[1]['impressions'], 6);
        $this->assertEqual($result[2]['ad_id'], 2);
        $this->assertEqual($result[2]['zone_id'], 1);
        $this->assertEqual($result[2]['impressions_requested'], 1);
        $this->assertEqual($result[2]['priority_factor'], 0.5);
        $this->assertEqual($result[2]['past_zone_traffic_fraction'], 0.88);
        $this->assertEqual($result[2]['impressions'], 2);
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Method to test the updatePriorities method.
     *
     * Requirements:
     * Test 1: Test with no Date registered in the service locator, ensure false returned.
     * Test 2: Test with no data in the database, ensure data is correctly stored.
     * Test 3: Test with previous test data in the database, ensure data is correctly stored.
     */
    function testUpdatePriorities()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oOperationInterval = &OperationInterval::singleton();
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        // Insert the data into the ad_zone_assoc table, as an ad is linked to a zone
        $query = "
            INSERT INTO 
                {$conf['table']['prefix']}{$conf['table']['ad_zone_assoc']}
                (
                    ad_id,
                    zone_id,
                    priority
                )
            VALUES
                (
                    1,
                    1,
                    0
                )";
        $dbh->query($query);
        
        // Test 1
        $oServiceLocator = &ServiceLocator::instance();
        $oServiceLocator->remove('now');
        $aData = 
            array(
                array(
                    'ads' => array(
                        array(
                            'ad_id'                      => '1',
                            'zone_id'                    => '1',
                            'impressions_requested'      => '1000',
                            'priority'                   => '0.45',
                            'priority_factor'            => null,
                            'past_zone_traffic_fraction' => null
                        )
                    )
                )
            );
        $result = $oMaxDalMaintenance->updatePriorities($aData);
        $this->assertFalse($result);
        
        // Test 2
        $oDate = &new Date();
        $oServiceLocator->register('now', $oDate);
        $result = $oMaxDalMaintenance->updatePriorities($aData);
        $this->assertTrue($result);
        $query = "
            SELECT
                ad_id,
                zone_id,
                priority
            FROM
                {$conf['table']['prefix']}{$conf['table']['ad_zone_assoc']}
            WHERE
                ad_id = 1";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['ad_id'], 1);
        $this->assertEqual($row['zone_id'], 1);
        $this->assertEqual($row['priority'], 0.45);
        $query = "
            SELECT
                operation_interval,
                operation_interval_id,
                interval_start,
                interval_end,
                ad_id,
                zone_id,
                impressions_requested,
                priority,
                priority_factor,
                past_zone_traffic_fraction,
                created,
                created_by,
                expired,
                expired_by
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_summary_ad_zone_assoc']}
            WHERE
                ad_id = 1";
        $row = $dbh->getRow($query);
        $currentOperationIntervalID = $oOperationInterval->getIntervalID($oDate);
        $aDates = $oOperationInterval->convertDateToOperationIntervalStartAndEndDates($oDate);
        $this->assertEqual($row['operation_interval'], $conf['maintenance']['operationInterval']);
        $this->assertEqual($row['operation_interval_id'], $currentOperationIntervalID);
        $this->assertEqual($row['interval_start'], $aDates['start']->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['interval_end'], $aDates['end']->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['ad_id'], 1);
        $this->assertEqual($row['zone_id'], 1);
        $this->assertEqual($row['impressions_requested'], 1000);
        $this->assertEqual($row['priority'], 0.45);
        $this->assertNull($row['priority_factor']);
        $this->assertNull($row['past_zone_traffic_fraction']);
        $this->assertEqual($row['created'], $oDate->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['created_by'], 0);
        $this->assertNull($row['expired']);
        $this->assertNull($row['expired_by']);
        
        // Test 3
        $aData =
            array(
                array(
                    'ads' => array(
                        array(
                            'ad_id'                      => 1,
                            'zone_id'                    => 1,
                            'impressions_requested'      => 2000,
                            'priority'                   => 0.9,
                            'priority_factor'            => 0.1,
                            'past_zone_traffic_fraction' => 0.99
                        ),
                        array(
                            'ad_id'                      => 2,
                            'zone_id'                    => 2,
                            'impressions_requested'      => 500,
                            'priority'                   => 0.1,
                            'priority_factor'            => 0.2,
                            'past_zone_traffic_fraction' => 0.45
                        )
                    )
                )
            );
        $oOldDate = &new Date();
        $oOldDate->copy($oDate);
        $oDate = &new Date();
        $oServiceLocator->register('now', $oDate);
        $result = $oMaxDalMaintenance->updatePriorities($aData);
        $this->assertTrue($result);
        $query = "
            SELECT
                ad_id,
                zone_id,
                priority
            FROM
                {$conf['table']['prefix']}{$conf['table']['ad_zone_assoc']}
            WHERE
                ad_id = 1";
        $row = $dbh->getRow($query);
        $this->assertEqual($row['ad_id'], 1);
        $this->assertEqual($row['zone_id'], 1);
        $this->assertEqual($row['priority'], 0.9);
        $query = "
            SELECT
                operation_interval,
                operation_interval_id,
                interval_start,
                interval_end,
                ad_id,
                zone_id,
                impressions_requested,
                priority,
                priority_factor,
                past_zone_traffic_fraction,
                created,
                created_by,
                expired,
                expired_by
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_summary_ad_zone_assoc']}
            WHERE
                ad_id = 1
                AND expired IS NOT NULL";
        $row = $dbh->getRow($query);
        $currentOperationIntervalID = $oOperationInterval->getIntervalID($oOldDate);
        $aDates = $oOperationInterval->convertDateToOperationIntervalStartAndEndDates($oOldDate);
        $this->assertEqual($row['operation_interval'], $conf['maintenance']['operationInterval']);
        $this->assertEqual($row['operation_interval_id'], $currentOperationIntervalID);
        $this->assertEqual($row['interval_start'], $aDates['start']->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['interval_end'], $aDates['end']->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['ad_id'], 1);
        $this->assertEqual($row['zone_id'], 1);
        $this->assertEqual($row['impressions_requested'], 1000);
        $this->assertEqual($row['priority'], 0.45);
        $this->assertNull($row['priority_factor']);
        $this->assertNull($row['past_zone_traffic_fraction']);
        $this->assertEqual($row['created'], $oOldDate->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['created_by'], 0);
        $this->assertEqual($row['expired'], $oDate->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['expired_by'], 0);
        $query = "
            SELECT
                operation_interval,
                operation_interval_id,
                interval_start,
                interval_end,
                ad_id,
                zone_id,
                impressions_requested,
                priority,
                priority_factor,
                past_zone_traffic_fraction,
                created,
                created_by,
                expired,
                expired_by
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_summary_ad_zone_assoc']}
            WHERE
                ad_id = 1
                AND expired IS NULL";
        $row = $dbh->getRow($query);
        $currentOperationIntervalID = $oOperationInterval->getIntervalID($oDate);
        $aDates = $oOperationInterval->convertDateToOperationIntervalStartAndEndDates($oDate);
        $this->assertEqual($row['operation_interval'], $conf['maintenance']['operationInterval']);
        $this->assertEqual($row['operation_interval_id'], $currentOperationIntervalID);
        $this->assertEqual($row['interval_start'], $aDates['start']->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['interval_end'], $aDates['end']->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['ad_id'], 1);
        $this->assertEqual($row['zone_id'], 1);
        $this->assertEqual($row['impressions_requested'], 2000);
        $this->assertEqual($row['priority'], 0.9);
        $this->assertEqual($row['priority_factor'], 0.1);
        $this->assertEqual($row['past_zone_traffic_fraction'], 0.99);
        $this->assertEqual($row['created'], $oDate->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['created_by'], 0);
        $this->assertNull($row['expired']);
        $this->assertNull($row['expired_by']);
        $query = "
            SELECT
                operation_interval,
                operation_interval_id,
                interval_start,
                interval_end,
                ad_id,
                zone_id,
                impressions_requested,
                priority,
                priority_factor,
                past_zone_traffic_fraction,
                created,
                created_by,
                expired,
                expired_by
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_summary_ad_zone_assoc']}
            WHERE
                ad_id = 2";
        $row = $dbh->getRow($query);
        $currentOperationIntervalID = $oOperationInterval->getIntervalID($oDate);
        $aDates = $oOperationInterval->convertDateToOperationIntervalStartAndEndDates($oDate);
        $this->assertEqual($row['operation_interval'], $conf['maintenance']['operationInterval']);
        $this->assertEqual($row['operation_interval_id'], $currentOperationIntervalID);
        $this->assertEqual($row['interval_start'], $aDates['start']->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['interval_end'], $aDates['end']->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['ad_id'], 2);
        $this->assertEqual($row['zone_id'], 2);
        $this->assertEqual($row['impressions_requested'], 500);
        $this->assertEqual($row['priority'], 0.1);
        $this->assertEqual($row['priority_factor'], 0.2);
        $this->assertEqual($row['past_zone_traffic_fraction'], 0.45);
        $this->assertEqual($row['created'], $oDate->format('%Y-%m-%d %H:%M:%S'));
        $this->assertEqual($row['created_by'], 0);
        $this->assertNull($row['expired']);
        $this->assertNull($row['expired_by']);
        TestEnv::restoreEnv();
    }
    
    /**
     * Tests obtaining the correct tracker to ad impression connection window.
     *
     * Requirements:
     * Test 1: Test without data in the table, ensure a value of zero returned
     * Test 2: Test with data in the table, but windows of zero, ensure a
     *         value of zero returned
     * Test 3: Test with data in the table, only the click window > 0, ensure
     *         a value of zero returned
     * Test 4: Test with data in the table, ensure the correct value returned
     */
    function testMaxAdImpressionWindow()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $da = MAX_Dal_Maintenance::singleton();
        
        // Test 1: No data
        $max = $da->maxAdImpressionWindow();
        $this->assertEqual($max, 0);
        
        // Start a transaction
        TestEnv::startTransaction();
        
        // Test 2: A tracker with impression and click windows of 0
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']}
                (
                    viewwindow,
                    clickwindow
                )
            VALUES
                (
                    0,
                    0
                )";
        $dbh->query($query);
        $max = $da->maxAdImpressionWindow();
        $this->assertEqual($max, 0);
        
        // Test 3: A tracker with impression window of 0 and click windows of 60
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']}
                (
                    viewwindow,
                    clickwindow
                )
            VALUES
                (
                    0,
                    60
                )";
        $dbh->query($query);
        $max = $da->maxAdImpressionWindow();
        $this->assertEqual($max, 0);
        
        // Test 4: A tracker with impression window of 180 and click windows of 60
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']}
                (
                    viewwindow,
                    clickwindow
                )
            VALUES
                (
                    180,
                    60
                )";
        $dbh->query($query);
        $max = $da->maxAdImpressionWindow();
        $this->assertEqual($max, 180);
        
        // Roll back the transaction
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Tests obtaining the correct tracker to ad click connection window.
     *
     * Requirements:
     * Test 1: Test without data in the table, ensure a value of zero returned
     * Test 2: Test with data in the table, but windows of zero, ensure a
     *         value of zero returned
     * Test 3: Test with data in the table, only the impression window > 0, ensure
     *         a value of zero returned
     * Test 4: Test with data in the table, ensure the correct value returned
     */
    function testMaxAdClickWindow()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $da = MAX_Dal_Maintenance::singleton();
        
        // Test 1
        $max = $da->maxAdClickWindow();
        $this->assertEqual($max, 0);
        
        // Start a transaction
        TestEnv::startTransaction();
        
        // Test 2
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']}
                (
                    viewwindow,
                    clickwindow
                )
            VALUES
                (
                    0,
                    0
                )";
        $dbh->query($query);
        $max = $da->maxAdClickWindow();
        $this->assertEqual($max, 0);
        
        // Test 3
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']}
                (
                    viewwindow,
                    clickwindow
                )
            VALUES
                (
                    60,
                    0
                )";
        $dbh->query($query);
        $max = $da->maxAdClickWindow();
        $this->assertEqual($max, 0);
        
        // Test 4
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']}
                (
                    viewwindow,
                    clickwindow
                )
            VALUES
                (
                    60,
                    180
                )";
        $dbh->query($query);
        $max = $da->maxAdClickWindow();
        $this->assertEqual($max, 180);
        
        // Roll back the transaction
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Test 1 - check values and format of return values
     *
     * @TODO Implement tests to check date range being applied correctly
     */
    function testGetZonesImpressionAverageByRange()
    {
        TestEnv::startTransaction();
        $dbh = &MAX_DB::singleton();
        // Set up test test data
        $conf = $GLOBALS['_MAX']['CONF'];
        // Write two record for same interval id one week apart
        $query = "INSERT INTO 
                    {$conf['table']['prefix']}{$conf['table']['data_summary_zone_impression_history']} 
                    (
                        data_summary_zone_impression_history_id, 
                        operation_interval,
                        operation_interval_id, 
                        interval_start, 
                        interval_end, 
                        zone_id,
                        forecast_impressions,
                        actual_impressions
                    )
                    VALUES
                    (1, 60, 0, '2005-05-02 00:00:00',  '2005-05-02 00:59:59',  1,  200,    400),
                    (2, 60, 0, '2005-05-09 00:00:00',  '2005-05-09 00:59:59',  1,  100,    200),
                    (3, 60, 1, '2005-05-02 01:00:00',  '2005-05-02 01:59:59',  1,  200,    500),
                    (4, 60, 1, '2005-05-09 01:00:00',  '2005-05-09 01:59:59',  1,  100,    700)";
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh->query($query);
        
        // Set up operation interval date range to query
        $oStartDate = new Date('2005-05-16 00:00:00');
        $oEndDate = new Date('2005-05-16 01:59:59');
        
        // Array of zone ids to resolve
        $aZones = array(1);
        
        // Number of weeks to get average over
        $weeks = 2;
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        $result = &$oMaxDalMaintenance->getZonesImpressionAverageByRange($aZones, $oStartDate, $oEndDate, $weeks);
        
        $this->assertEqual(count($result), 1);
        $this->assertEqual(count($result[1]), 2);
        
        $this->assertEqual($result[1][0]['zone_id'], 1);
        $this->assertEqual($result[1][0]['operation_interval_id'], 0);
        $this->assertEqual($result[1][0]['average_impressions'], 300);
        
        $this->assertEqual($result[1][1]['zone_id'], 1);
        $this->assertEqual($result[1][1]['operation_interval_id'], 1);
        $this->assertEqual($result[1][1]['average_impressions'], 600);
        
        TestEnv::rollbackTransaction();
    }
    
    /**
     * Test 1 - check values and format of return values
     *
     * @TODO Implement tests to check date range being applied correctly
     */
    function testGetZonesImpressionHistoryByRange()
    {
        TestEnv::startTransaction();
        $dbh = &MAX_DB::singleton();
        // set up test test data
        $conf = $GLOBALS['_MAX']['CONF'];
        // write two record for same interval id one week apart
        $query = "INSERT INTO 
                    {$conf['table']['prefix']}{$conf['table']['data_summary_zone_impression_history']} 
                    (
                        data_summary_zone_impression_history_id, 
                        operation_interval,
                        operation_interval_id, 
                        interval_start, 
                        interval_end, 
                        zone_id,
                        forecast_impressions,
                        actual_impressions
                    )
                    VALUES
                    (1,  60, 0,  '2005-05-09 00:00:00',  '2005-05-09 00:59:59',  1,  100,   700),
                    (2,  60, 1,  '2005-05-09 01:00:00',  '2005-05-09 01:59:59',  1,  200,   300),
                    (3,  60, 2,  '2005-05-09 02:00:00',  '2005-05-09 02:59:59',  1,  300,   400),
                    (4,  60, 3,  '2005-05-09 03:00:00',  '2005-05-09 03:59:59',  1,  500,   200),
                    (5,  60, 4,  '2005-05-09 04:00:00',  '2005-05-09 04:59:59',  1,  500,   600),
                    (6,  60, 5,  '2005-05-09 05:00:00',  '2005-05-09 05:59:59',  1,  600,   700)";
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh->query($query);
        
        // Set up operation interval date range to query
        $oStartDate = new Date('2005-05-09 01:00:00');
        $oEndDate = new Date('2005-05-09 04:00:00');
        
        // Array of zone ids to resolve
        $aZones = array(1);
        
        // Number of weeks to get average over
        $weeks = 2;
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        $result = &$oMaxDalMaintenance->getZonesImpressionHistoryByRange($aZones, $oStartDate, $oEndDate);
        
        $this->assertEqual(count($result), 1);
        $this->assertEqual(count($result[1]), 4);
        $this->assertEqual($result[1][1]['zone_id'], 1);
        $this->assertEqual($result[1][1]['operation_interval_id'], 1);
        $this->assertEqual($result[1][1]['forecast_impressions'], 200);
        
        $this->assertEqual($result[1][1]['actual_impressions'], 300);
        $this->assertEqual($result[1][2]['zone_id'], 1);
        $this->assertEqual($result[1][2]['operation_interval_id'], 2);
        $this->assertEqual($result[1][2]['forecast_impressions'], 300);
        $this->assertEqual($result[1][2]['actual_impressions'], 400);
        
        $this->assertEqual($result[1][3]['zone_id'], 1);
        $this->assertEqual($result[1][3]['operation_interval_id'], 3);
        $this->assertEqual($result[1][3]['forecast_impressions'], 500);
        $this->assertEqual($result[1][3]['actual_impressions'], 200);
        
        $this->assertEqual($result[1][4]['zone_id'], 1);
        $this->assertEqual($result[1][4]['operation_interval_id'], 4);
        $this->assertEqual($result[1][4]['forecast_impressions'], 500);
        $this->assertEqual($result[1][4]['actual_impressions'], 600);
        
        TestEnv::rollbackTransaction();
    }
    
    /**
     * A method to test the saveZoneImpressionForecasts method.
     */
    function testSaveZoneImpressionForecasts()
    {
        TestEnv::startTransaction();
        $dbh = &MAX_DB::singleton();
        // Test data 
        $aForecasts = array(
            1 => array(
                0 => array('forecast_impressions' => 100, 'interval_start' => '2005-05-20 00:00:00', 'interval_end' => '2005-05-20 00:59:59'),
                1 => array('forecast_impressions' => 200, 'interval_start' => '2005-05-20 01:00:00', 'interval_end' => '2005-05-20 01:59:59'),
                2 => array('forecast_impressions' => 300, 'interval_start' => '2005-05-20 02:00:00', 'interval_end' => '2005-05-20 02:59:59'),
            ),
            2 => array(
                0 => array('forecast_impressions' => 200, 'interval_start' => '2005-05-20 00:00:00', 'interval_end' => '2005-05-20 00:59:59'),
                1 => array('forecast_impressions' => 400, 'interval_start' => '2005-05-20 01:00:00', 'interval_end' => '2005-05-20 01:59:59'),
                2 => array('forecast_impressions' => 600, 'interval_start' => '2005-05-20 02:00:00', 'interval_end' => '2005-05-20 02:59:59'),
            )
        );
        
        // Run write method
        $oMaxDalMaintenance = MAX_Dal_Maintenance::singleton();
        $oMaxDalMaintenance->saveZoneImpressionForecasts($aForecasts);
        
        // Test keys
        $conf = $GLOBALS['_MAX']['CONF'];
        $query = 'SELECT * from ' . $conf['table']['prefix'] . $conf['table']['data_summary_zone_impression_history'];
        $dbh->query($query);
        $row = $dbh->getAll($query);
        $this->assertTrue(isset($row[0]['data_summary_zone_impression_history_id']));
        $this->assertTrue(isset($row[0]['operation_interval']));
        $this->assertTrue(isset($row[0]['operation_interval_id']));
        $this->assertTrue(isset($row[0]['interval_start']));
        $this->assertTrue(isset($row[0]['interval_end']));
        $this->assertTrue(isset($row[0]['zone_id']));
        $this->assertTrue(isset($row[0]['forecast_impressions']));
        
        // Test forecast values written
        foreach($row as $key => $aValues) {
            $this->assertTrue($aValues['forecast_impressions'] > 0);
            $this->assertTrue(!(empty($aValues['interval_start'])));
            $this->assertTrue(!(empty($aValues['interval_end'])));
            // Bit funny way to test value, done so I can test in loop
            $this->assertEqual($aValues['forecast_impressions'], (($aValues['operation_interval_id'] + 1) * ($aValues['zone_id'] * 100)));
        }
        TestEnv::rollbackTransaction();
    }
    
    /**
     * A method to test the getActiveZones method.
     */
    function testGetActiveZones()
    {
        $da = MAX_Dal_Maintenance::singleton();
        TestEnv::startTransaction();
        $this->_generateStatsTwo();
        $ret = $da->getActiveZones();
                
        $this->assertTrue(is_array($ret));
        $this->assertTrue(count($ret) == 1);
        $zone = $ret[0];
        $this->assertTrue(array_key_exists('zoneid', $zone));
        $this->assertTrue(array_key_exists('zonename', $zone));
        $this->assertTrue(array_key_exists('zonetype', $zone));
        TestEnv::rollbackTransaction();
    }
    
    /**
     * A method to test the getAdsByPlacementId method.
     */
    function testGetAdsByPlacementId()
    {
        $da = MAX_Dal_Maintenance::singleton();
        TestEnv::startTransaction();
        $this->_generateStatsOne();
        $ret = $da->getAdsByPlacementId(1);
        
        $this->assertTrue(is_array($ret));
        $this->assertTrue(count($ret) == 2);
        $advert = $ret[0];
        $this->assertTrue(array_key_exists('ad_id', $advert));
        $this->assertTrue(array_key_exists('type', $advert));
        $this->assertTrue(array_key_exists('weight', $advert));
        $this->assertTrue(array_key_exists('active', $advert));
        TestEnv::rollbackTransaction();
    }
    
    /**
     * A method to test the setRequiredAdImpressions method.
     */
    function testSaveRequiredAdImpressions()
    {
        $oDal = MAX_Dal_Maintenance::singleton();
        $oTable = &MAX_Table_Priority::singleton();
        $oTable->createTable('tmp_ad_required_impression');
        $aData = array(
            array(
                'ad_id'                => 23,
                'required_impressions' => 140,
            )
        );
        $result = $oDal->saveRequiredAdImpressions($aData);
        $dbh = & MAX_DB::singleton();
        $query = "SELECT * FROM tmp_ad_required_impression";
        $result = $dbh->getAll($query);
        $this->assertTrue(is_array($result));
        $this->assertTrue(count($result) == 1);
        $item = $result[0];
        $this->assertTrue(array_key_exists('ad_id', $item));
        $this->assertTrue(array_key_exists('required_impressions', $item));
        $this->assertEqual($item['ad_id'], 23);
        $this->assertEqual($item['required_impressions'], 140);
        TestEnv::restoreEnv();
    }
    
    /**
     * A method to test the getRequiredAdImpressions method.
     */
    function testGetRequiredAdImpressions()
    {
        $oDal = MAX_Dal_Maintenance::singleton();
        $oTable = &MAX_Table_Priority::singleton();
        $oTable->createTable('tmp_ad_required_impression');
        $aData = array(
            array(
                'ad_id'                => 23,
                'required_impressions' => 140,
            ),
            array(
                'ad_id'                => 29,
                'required_impressions' => 120,
            )
        );
        $result = $oDal->saveRequiredAdImpressions($aData);
        $aAdvertID = array(
            1,
            2,
            29
        );
        $aData = $oDal->getRequiredAdImpressions($aAdvertID);
        $this->assertEqual(count($aData), 1);
        $this->assertTrue(array_key_exists(29, $aData));
        $this->assertEqual($aData[29], 120);
        TestEnv::restoreEnv();
    }
    
    /**
     * A method to test the getZoneImpressionForecasts method.
     *
     * @TODO Not implemented.
     */
    function testGetZoneImpressionForecasts() {}
    
    /**
     * A method to test the getAdZoneAssociationsByAds method.
     *
     * @TODO Not implemented.
     */
    function testGetAdZoneAssociationsByAds() {}
    
    /**
     * A method to test the saveAllocatedImpressions method.
     */
    function testSaveAllocatedImpressions()
    {
        $dbh = &MAX_DB::singleton();
        $oDal = MAX_Dal_Maintenance::singleton();
        // Create the required temporary table for the tests
        $oTable = &MAX_Table_Priority::singleton();
        $oTable->createTable('tmp_ad_zone_impression');
        // Prepare the test data
        $aData = array(
            array(
                'ad_id'       => 56,
                'zone_id'     => 11,
                'impressions' => 9997
            ),
            array(
                'ad_id'       => 56,
                'zone_id'     => 12,
                'impressions' => 24
            )
        );
        $result = $oDal->saveAllocatedImpressions($aData);
        $query = "SELECT * FROM tmp_ad_zone_impression ORDER BY ad_id, zone_id";
        $result = $dbh->getAll($query);
        $this->assertTrue(is_array($result));
        $this->assertTrue(count($result) == 2);
        $tmp = $result[0];
        $this->assertTrue(array_key_exists('ad_id', $tmp));
        $this->assertEqual($tmp['ad_id'], 56);
        $this->assertTrue(array_key_exists('zone_id', $tmp));
        $this->assertEqual($tmp['zone_id'], 11);
        $this->assertTrue(array_key_exists('impressions', $tmp));
        $this->assertEqual($tmp['impressions'], 9997);
        $tmp = $result[1];
        $this->assertTrue(array_key_exists('ad_id', $tmp));
        $this->assertEqual($tmp['ad_id'], 56);
        $this->assertTrue(array_key_exists('zone_id', $tmp));
        $this->assertEqual($tmp['zone_id'], 12);
        $this->assertTrue(array_key_exists('impressions', $tmp));
        $this->assertEqual($tmp['impressions'], 24);
        TestEnv::restoreEnv();
    }
    
    /**
     * A method to test the getAllActiveAgencyIds method.
     */
    function testGetAllActiveAgencyIds()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &MAX_DB::singleton();
        $oDal = MAX_Dal_Maintenance::singleton();
        // Prepare the test data
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table']['agency']}
                    (
                        agencyid,
                        active
                    )
                VALUES
                    (
                        1,
                        1
                    ),
                    (
                        2,
                        0
                    ),
                    (
                        3,
                        1
                    )";
        $dbh->query($query);
        // Test the method
        $result = $oDal->getAllActiveAgencyIds();
        $this->assertTrue(is_array($result));
        $this->assertEqual(count($result), 2);
        $this->assertEqual($result[0], 1);
        $this->assertEqual($result[1], 3);
        TestEnv::restoreEnv();
    }
    
    /**
     * A method for testing the obtainPriorityLock and
     * releasePriorityLock methods.
     *
     * @TODO Complete testing using a separate client connection to
     *       ensure locking works.
     */
    function testLocking()
    {
        $dbh = &MAX_DB::singleton();
        $oDal = MAX_Dal_Maintenance::singleton();
        // Try to get the lock
        $result = $oDal->obtainPriorityLock();
        $this->assertTrue($result);
        // Try to get the lock again, with a brand new connection,
        // and ensure that the lock is NOT obtained
        
        // Release the lock
        $result = $oDal->releasePriorityLock();
        $this->assertTrue($result);
        // Try to get the lock again with the new connection, and
        // ensure the lock IS obtained

        // Release the lock from the new connection
        
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    function xtestGetAdLimitations()
    {
        $da = MAX_Dal_Maintenance::singleton();
        TestEnv::startTransaction();
        $this->_generateStatsOne();
        $campaignId = 2;
        $wheres[] = array('banners.campaignid = '.$campaignId, 'AND');
        $joins[] = array('banners','banners.bannerid=acls.bannerid');
        $ret = $da->getAdLimitations(array(), $wheres, $joins);
        
        $this->assertTrue(is_array($ret));
        $this->assertTrue(count($ret) == 4);
        $limit = $ret[0];
        $this->assertTrue(array_key_exists('bannerid', $limit));
        $this->assertTrue(array_key_exists('logical', $limit));
        $this->assertTrue(array_key_exists('type', $limit));
        $this->assertTrue(array_key_exists('comparison', $limit));
        $this->assertTrue(array_key_exists('data', $limit));
        $this->assertTrue(array_key_exists('executionorder', $limit));
        
        TestEnv::rollbackTransaction();
    }
    
    function xtestGetCampaignsWithParams()
    {
        $da = MAX_Dal_Maintenance::singleton();
        TestEnv::startTransaction();
        $this->_generateStatsOne();
        
        $conf = $GLOBALS['_MAX']['CONF'];
        $banners = $conf['table']['prefix'] . $conf['table']['banners'];
        $campaigns = $conf['table']['prefix'] . $conf['table']['campaigns'];
        
        $fields[] = "$banners.bannerid";
        $fields[] = "$banners.weight";
        $wheres[] = array("$campaigns.campaignid = $banners.campaignid", 'AND');
        $joins[] = array($banners,"$banners.active='t'");
        $ret = $da->getPlacements($fields, $wheres, $joins);

        //[0] => Array
        //    (
        //        [campaignid] => 1
        //        [views] => 0
        //        [clicks] => 400
        //        [conversions] => 0
        //        [expire] => 2005-05-23
        //        [target_impression] => 0
        //        [target_click] => 0
        //        [target_conversion] => 0
        //        [priority] => 3
        //        [_banners_bannerid] => 1
        //        [_banners_weight] => 1
        //    )
        //
        //[1] => Array
          
        $this->assertTrue(is_array($ret));
        $this->assertTrue(count($ret) == 6);
        $campaign = $ret[0];
        $this->assertTrue(array_key_exists('campaignid', $campaign));
        $this->assertTrue(array_key_exists('views', $campaign));
        $this->assertTrue(array_key_exists('clicks', $campaign));
        $this->assertTrue(array_key_exists('conversions', $campaign));
        $this->assertTrue(array_key_exists('expire', $campaign));
        $this->assertTrue(array_key_exists('target_impression', $campaign));
        $this->assertTrue(array_key_exists('target_click', $campaign));
        $this->assertTrue(array_key_exists('target_conversion', $campaign));
        $this->assertTrue(array_key_exists('priority', $campaign));
        TestEnv::rollbackTransaction();
    }
    
    
    function xtestGetCampaignsOrdered()
    {
        $da = MAX_Dal_Maintenance::singleton();
        TestEnv::startTransaction();
        $this->_generateStatsOne();
        $orderBys[] = array('priority','DESC');
        $orderBys[] = array('expire','ASC');
        $ret = $da->getPlacements(array(), array(), array(), $orderBys);
        
        $this->assertTrue(is_array($ret));
        $this->assertTrue(count($ret) == 5);
        $campaign = $ret[0];
        $this->assertTrue(array_key_exists('campaignid', $campaign));
        $this->assertTrue(array_key_exists('views', $campaign));
        $this->assertTrue(array_key_exists('clicks', $campaign));
        $this->assertTrue(array_key_exists('conversions', $campaign));
        $this->assertTrue(array_key_exists('expire', $campaign));
        $this->assertTrue(array_key_exists('target_impression', $campaign));
        $this->assertTrue(array_key_exists('target_click', $campaign));
        $this->assertTrue(array_key_exists('target_conversion', $campaign));
        $this->assertTrue(array_key_exists('priority', $campaign));
        
        //  test ordering
        $this->assertTrue($ret[0]['priority'] > $ret[4]['priority']);
        TestEnv::rollbackTransaction();
    }
    
    function xtestGetAdConstraints()
    {
        $da = MAX_Dal_Maintenance::singleton();
        TestEnv::startTransaction();
        $this->_generateStatsOne();
        $wheres[] = array(  $conf['table']['prefix'] . 
                            $conf['table']['acls'].'bannerid = 3', 'AND');
		$ret = $da->getAdLimitations(array(), $wheres);
		
        $this->assertTrue(is_array($ret));
        $this->assertTrue(count($ret) == 2);
        $acls = $ret[0];
        $this->assertTrue(array_key_exists('bannerid', $acls));
        $this->assertTrue(array_key_exists('logical', $acls));
        $this->assertTrue(array_key_exists('type', $acls));
        $this->assertTrue(array_key_exists('comparison', $acls));
        $this->assertTrue(array_key_exists('data', $acls));
        $this->assertTrue(array_key_exists('executionorder', $acls));
        TestEnv::rollbackTransaction();
    }
    
    /**
     * A method to generate data for testing.
     *
     * @access private
     */
    function _generateStatsOne()
    {
        $conf = & $GLOBALS['_MAX']['CONF'];
        $dbh = & MAX_DB::singleton();
        
        $date = new Date();
        $date->addSeconds((SECONDS_PER_DAY * 4));
        $expiryDate1 = $date->getYear() . "-" . $date->getMonth() . "-" . $date->getDay();
        
        $date->addSeconds((SECONDS_PER_DAY * 4));
        $expiryDate2 = $date->getYear() . "-" . $date->getMonth() . "-" . $date->getDay();        
        
        $date->subtractSeconds((SECONDS_PER_DAY * 2));
        $expiryDateLessTwoDays = $date->getYear() . "-" . $date->getMonth() . "-" . $date->getDay();
        
        //  Add 3 campaigns
        $campaignsTable = $conf['table']['prefix'] . 'campaigns';
        $sql = "INSERT INTO $campaignsTable
        (campaignid, campaignname,      clientid,   views,  clicks, conversions,    expire,             activate,       active,     priority,   weight, target_impression, optimise,   anonymous)
        VALUES (1,   'test  campaign',  1,          0,      400,      0,          '" . $expiryDate1 . "',  '0000-00-00',   't',        '3',        1,      0,      'f',        'f')";
        $result = $dbh->query($sql);

        $sql = "INSERT INTO $campaignsTable
        (campaignid, campaignname,      clientid,   views,  clicks, conversions,    expire,             activate,       active,     priority,   weight, target_impression, optimise,   anonymous)
        VALUES (2,   'test  campaign',  1,          0,      0,        400,          '0000-00-00',         '0000-00-00',   't',         '2',        1,      0,      'f',        'f')";
        $result = $dbh->query($sql);

        $sql = "INSERT INTO $campaignsTable
        (campaignid, campaignname,      clientid,   views,  clicks, conversions,    expire,             activate,       active,     priority,   weight, target_impression, optimise,   anonymous)
        VALUES (3,   'test  campaign',  1,          500,      0,      0,            '" . $expiryDate2 . "', '0000-00-00',   't',       '3',        1,      0,      'f',        'f')";
        $result = $dbh->query($sql);
        
        $sql = "INSERT INTO $campaignsTable
        (campaignid, campaignname,      clientid,   views,  clicks, conversions,    expire,             activate,       active,     priority,   weight, target_impression, optimise,   anonymous)
        VALUES (4,   'test  campaign',  1,          500,      0,      401,          '0000-00-00',  '0000-00-00',   't',                '4',        2,      0,      'f',        'f')";
        $result = $dbh->query($sql);
        
        $sql = "INSERT INTO $campaignsTable
        (campaignid, campaignname,      clientid,   views,  clicks, conversions,    expire,             activate,       active,     priority,   weight, target_impression, optimise,   anonymous)
        VALUES (5,   'test  campaign',  1,          500,      0,      401,          '0000-00-00',  '0000-00-00',   't',                '3',        2,      0,      'f',        'f')";
        $result = $dbh->query($sql);  




        //  add a banner to campaign 1
        $bannersTable = $conf['table']['prefix'] . 'banners';
        $sql = "INSERT INTO $bannersTable (bannerid, campaignid, active, contenttype, pluginversion, storagetype, filename, imageurl, htmltemplate, htmlcache, width, height, weight, seq, target, url, alt, status, bannerTEXT, description, autohtml, adserver, block, capping, session_capping, compiledlimitation, append, appendtype, bannertype, alt_filename, alt_imageurl, alt_contenttype)
        VALUES (1,1,'t','gif',0,'sql','468x60_4.gif','','','',468,60,1,0,'','http://www.google.com','Campaign1 - Banner 1','','','','f','',0, 0,0,'','',0,0,'','','');";
        $result = $dbh->query($sql);

        // add Campaign 1 - banner 1 - acls delivery restrictions
        $aclsTable = $conf['table']['prefix'] . 'acls';
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,                        executionorder)
                                VALUES (1,          'and',   'date',   '!=',       '$expiryDateLessTwoDays',    0)";
        $result = $dbh->query($sql);
        
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,   executionorder)
                                VALUES (1,          'and',   'time',   '!=',       '1',    1)";
        $result = $dbh->query($sql);
        
        // add 2nd banner to campaign 1
        $bannersTable = $conf['table']['prefix'] . 'banners';
        $sql = "INSERT INTO $bannersTable (bannerid, campaignid, active, contenttype, pluginversion, storagetype, filename, imageurl, htmltemplate, htmlcache, width, height, weight, seq, target, url, alt, status, bannerTEXT, description, autohtml, adserver, block, capping, session_capping, compiledlimitation, append, appendtype, bannertype, alt_filename, alt_imageurl, alt_contenttype)
        VALUES (2,1,'t','gif',0,'sql','468x60_4.gif','','','',468,60,1,0,'','http://www.google.com','Campaign 1 - Banner 2','','','','f','',0, 0,0,'','',0,0,'','','');";
        $result = $dbh->query($sql);

        // add Campaign 1 - banner 2 - acls delivery restrictions
        $aclsTable = $conf['table']['prefix'] . 'acls';
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,                        executionorder)
                                VALUES (2,          'and',   'date',   '!=',       '$expiryDateLessTwoDays',    0)";
        $result = $dbh->query($sql);
        
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,   executionorder)
                                VALUES (2,          'and',   'time',   '!=',       '1,2',    1)";
        $result = $dbh->query($sql);

        
        
        //  add 1st banner to campaign 2
        $bannersTable = $conf['table']['prefix'] . 'banners';
        $sql = "INSERT INTO $bannersTable (bannerid, campaignid, active, contenttype, pluginversion, storagetype, filename, imageurl, htmltemplate, htmlcache, width, height, weight, seq, target, url, alt, status, bannerTEXT, description, autohtml, adserver, block, capping, session_capping, compiledlimitation, append, appendtype, bannertype, alt_filename, alt_imageurl, alt_contenttype)
        VALUES (3,2,'t','gif',0,'sql','468x60_4.gif','','','',468,60,1,0,'','http://www.google.com','Campaign 2 - Banner 1','','','','f','',0, 0,0,'','',0,0,'','','');";
        $result = $dbh->query($sql);

        // add 1st banner - Campaign 2 - acls delivery restrictions
        $aclsTable = $conf['table']['prefix'] . 'acls';
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,                        executionorder)
                                VALUES (3,          'and',   'date',   '!=',       '$expiryDateLessTwoDays',    0)";
        $result = $dbh->query($sql);
        
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,   executionorder)
                                VALUES (3,          'and',   'weekday',   '!=',       '5',    1)";
        $result = $dbh->query($sql);
        
        
        // add 2nd banner to campaign 2
        $bannersTable = $conf['table']['prefix'] . 'banners';
        $sql = "INSERT INTO $bannersTable (bannerid, campaignid, active, contenttype, pluginversion, storagetype, filename, imageurl, htmltemplate, htmlcache, width, height, weight, seq, target, url, alt, status, bannerTEXT, description, autohtml, adserver, block, capping, session_capping, compiledlimitation, append, appendtype, bannertype, alt_filename, alt_imageurl, alt_contenttype)
        VALUES (4,2,'t','gif',0,'sql','468x60_4.gif','','','',468,60,1,0,'','http://www.google.com','Campaign 2 - Banner 2','','','','f','',0, 0,0,'','',0,0,'','','');";
        $result = $dbh->query($sql);

        // add banner 2 - Campaign 1 - acls delivery restrictions
        $aclsTable = $conf['table']['prefix'] . 'acls';
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,                        executionorder)
                                VALUES (4,          'or',   'date',   '!=',       '$expiryDateLessTwoDays',    0)";
        $result = $dbh->query($sql);
        
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,   executionorder)
                                VALUES (4,          'or',   'time',   '==',       '1,2',    1)";
        $result = $dbh->query($sql);
        
        
        //  add 1st banner to campaign 3
        $bannersTable = $conf['table']['prefix'] . 'banners';
        $sql = "INSERT INTO $bannersTable (bannerid, campaignid, active, contenttype, pluginversion, storagetype, filename, imageurl, htmltemplate, htmlcache, width, height, weight, seq, target, url, alt, status, bannerTEXT, description, autohtml, adserver, block, capping, session_capping, compiledlimitation, append, appendtype, bannertype, alt_filename, alt_imageurl, alt_contenttype)
        VALUES (5,3,'t','gif',0,'sql','468x60_4.gif','','','',468,60,1,0,'','http://www.google.com','Campaign 3 - Banner 1','','','','f','',0, 0,0,'','',0,0,'','','');";
        $result = $dbh->query($sql);

        // add 1st banner - Campaign 2 - acls delivery restrictions
        $aclsTable = $conf['table']['prefix'] . 'acls';
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,                        executionorder)
                                VALUES (5,          'and',   'date',   '!=',       '$expiryDateLessTwoDays',    0)";
        $result = $dbh->query($sql);
        
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,   executionorder)
                                VALUES (5,          'or',   'weekday',   '==',       '5',    1)";
        $result = $dbh->query($sql);
        
        
        
        // add 2nd banner to campaign 2
        $bannersTable = $conf['table']['prefix'] . 'banners';
        $sql = "INSERT INTO $bannersTable (bannerid, campaignid, active, contenttype, pluginversion, storagetype, filename, imageurl, htmltemplate, htmlcache, width, height, weight, seq, target, url, alt, status, bannerTEXT, description, autohtml, adserver, block, capping, session_capping, compiledlimitation, append, appendtype, bannertype, alt_filename, alt_imageurl, alt_contenttype)
        VALUES (6,3,'t','gif',0,'sql','468x60_4.gif','','','',468,60,1,0,'','http://www.google.com','Campaign 3 - Banner 2','','','','f','',0, 0,0,'','',0,0,'','','');";
        $result = $dbh->query($sql);

        // add banner 2 - Campaign 1 - acls delivery restrictions
        $aclsTable = $conf['table']['prefix'] . 'acls';
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,                        executionorder)
                                VALUES (6,          'or',   'date',   '!=',       '$expiryDateLessTwoDays',    0)";
        $result = $dbh->query($sql);
        
        $sql = "INSERT INTO $aclsTable (bannerid,   logical, type,     comparison, data,   executionorder)
                                VALUES (6,          'or',   'time',   '!=',       '1,2',    1)";
        $result = $dbh->query($sql);
    }
    
    /**
     * A method to generate data for testing.
     *
     * @access private
     */
    function _generateStatsTwo()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = & MAX_DB::singleton();
        
        // Populate data_summary_ad_hourly
        $statsTable = $conf['table']['prefix'] . 'data_summary_ad_hourly';
        for ($hour = 0; $hour < 24; $hour ++) {
            $sql = "INSERT INTO $statsTable VALUES(
                '',
                NOW(), 
                $hour, 
                1, -- banner id
                ".rand(1, 999).",
                ".rand(1, 999).",
                ".rand(1, 999).",
                ".rand(1, 999).",
                ".rand(1, 999).",
                ".rand(1, 999).",
                0
                )";
            $result = $dbh->query($sql);
        }
        // Populate campaigns table
        $campaignsTable = $conf['table']['prefix'] . 'campaigns';        
        $sql = "INSERT INTO $campaignsTable
        (campaignid, campaignname,      clientid,   views,  clicks, conversions,    expire,             activate,       active,     priority,   weight, target_impression, optimise,   anonymous)
        VALUES (4,   'test  campaign',  1,          500,      0,      401,          '0000-00-00',  '0000-00-00',   't',                '4',        2,      0,      'f',        'f')";
        $result = $dbh->query($sql);

        // Add a banner
        $bannersTable = $conf['table']['prefix'] . 'banners';
        $sql = "
            INSERT INTO
                $bannersTable
                    (
                        bannerid,
                        campaignid,
                        active,
                        contenttype,
                        pluginversion,
                        storagetype,
                        filename,
                        imageurl,
                        htmltemplate,
                        htmlcache,
                        width,
                        height,
                        weight,
                        seq,
                        target,
                        url,
                        alt,
                        status,
                        bannertext,
                        description,
                        autohtml,
                        adserver,
                        block,
                        capping,
                        session_capping,
                        compiledlimitation,
                        append,
                        appendtype,
                        bannertype,
                        alt_filename,
                        alt_imageurl,
                        alt_contenttype
                    )
                VALUES
                    (
                        1,
                        1,
                        't',
                        'txt',
                        0,
                        'txt',
                        '',
                        '',
                        '',
                        '',
                        0,
                        0,
                        1,
                        0,
                        '',
                        'http://exapmle.com',
                        '',
                        'asdf',
                        'tyerterty',
                        '',
                        'f',
                        '',
                        0,
                        0,
                        0,
                        'phpAds_aclCheckDate(\'20050502\', \'!=\') and phpAds_aclCheckClientIP(\'2.22.22.2\', \'!=\') and phpAds_aclCheckLanguage(\'(sq)|(eu)|(fo)|(fi)\', \'!=\')',
                        '',
                        0,
                        0,
                        '',
                        '',
                        ''
                    )";
        $result = $dbh->query($sql);
        
        // Add an agency record
        $agencyTable = $conf['table']['prefix'] . 'agency';
        $sql = "INSERT INTO $agencyTable VALUES (1, 'test agency', 'sdfsdfsdf', 'demian@phpkitchen.com', 'Demian Turner', 'passwd', 0, 'en_GB', 'logout.php', 1)";
        $result = $dbh->query($sql);

        // Add a client record (advertiser)
        $clientsTable = $conf['table']['prefix'] . 'clients';
        $sql = "
            INSERT INTO
                $clientsTable
                    (
                        clientid,
                        agencyid,
                        clientname,
                        contact,
                        email,
                        clientusername,
                        clientpassword,
                        permissions,
                        language,
                        report,
                        reportinterval,
                        reportlastdate,
                        reportdeactivate
                    )
                VALUES
                    (
                        1,
                        1,
                        'test client',
                        'yes',
                        'demian@phpkitchen.com',
                        'Demian Turner',
                        '',
                        59,
                        '',
                        't',
                        7,
                        '2005-03-21',
                        't'
                    )";
        $result = $dbh->query($sql);

        // Add an affiliate (publisher) record
        $publisherTable = $conf['table']['prefix'] . 'affiliates';
        $sql = "
            INSERT INTO
                $publisherTable
                    (
                        affiliateid,
                        agencyid,
                        name,
                        mnemonic,
                        contact,
                        email,
                        website,
                        username,
                        password,
                        permissions,
                        language,
                        publiczones
                    )
                VALUES
                    (
                        1,
                        1,
                        'test publisher',
                        'ABC',
                        'foo bar',
                        'foo@example.com',
                        'www.example.com',
                        'foo',
                        'bar',
                        NULL,
                        NULL,
                        'f'
                    )";
        $result = $dbh->query($sql);

        // Add zone record
        $zonesTable = $conf['table']['prefix'] . 'zones';
        $sql = "
            INSERT INTO
                $zonesTable
                    (
                        zoneid,
                        affiliateid,
                        zonename,
                        description,
                        delivery,
                        zonetype,
                        category,
                        width,
                        height,
                        ad_selection,
                        chain,
                        prepend,
                        append,
                        appendtype,
                        forceappend
                    )
                VALUES
                    (
                        1,
                        1,
                        'Demian Turner - Default',
                        '',
                        0,
                        3,
                        '',
                        728,
                        90,
                        '',
                        '',
                        '',
                        '',
                        0,
                        'f'
                    )";
        $result = $dbh->query($sql);
        
        // Add ad_zone_assoc record
        $zonesAssocTable = $conf['table']['prefix'] . 'ad_zone_assoc';
        $sql = "INSERT INTO $zonesAssocTable VALUES (1, 1, 1, '0', 1)";
        $result = $dbh->query($sql);
    }
    
}

?>
