<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: trackeranalysis.plugin.php 3702 2005-09-29 10:15:45Z radek $
*/

/**
 * @package    Plugins
 * @subpackage Reports
 * @author     Arlen Coupland <arlen@m3.net>
 */

require_once MAX_PATH . '/plugins/reports/Reports.php';
require_once MAX_PATH . '/plugins/reports/lib.php';

class Plugins_Reports_Advertiser_Trackeranalysis extends Plugins_Reports {
    
    // Public info function
    function info()
    {
    	include_once MAX_PATH . '/lib/max/Plugins/PluginTranslation.php';
        Max_PluginTranslation::init($this->module, $this->package);
        
        $plugininfo = array (
    		"plugin-name"			=> Max_PluginTranslation::translate('Conversion Tracking Analysis', $this->module, $this->package),
    		"plugin-description"	=> Max_PluginTranslation::translate('This report shows a breakdown of all conversion tracking activity.', $this->module, $this->package),
    		'plugin-category'       => 'advertiser',
    		'plugin-category-name'	=> Max_PluginTranslation::translate('Advertiser Reports', $this->module, $this->package),
    		"plugin-author"		    => "Arlen Coupland",
    		"plugin-export"		    => "csv",
    		"plugin-authorize"	    => phpAds_Admin+phpAds_Agency+phpAds_Client,
	        "plugin-import"		    => array (
            			            "clientid"	=> array (
                                				       "title"		=> $GLOBALS['strClient'],
                                				       "type"		=> "clientid-dropdown" 
                            				       ),
                            		"startdate"	=> array (
                                                       "title"	 => "Start Date",
                                                       "type"	 => "edit",
                                                       "size"	 => 10,
                                                       "default" => date("Y/m/d",mktime (0,0,0,date("m")-1,1,  date("Y")))
                                                   ),
                                    "enddate"	=> array (
                                                       "title"	=> "End Date",
                                                       "type"	=> "edit",
                                                       "size"	=> 10,
                                                       "default" => date("Y/m/d",mktime (0,0,0,date("m"),1,  date("Y")))
                                                   ),
    	                            )
        );
    	
    	return ($plugininfo);
    }
    
    /*-------------------------------------------------------*/
    /* Private plugin function                               */
    /*-------------------------------------------------------*/
    function execute($trackerid, $startdate, $enddate)
    {
        include_once MAX_PATH . '/plugins/reports/ReportGenerator.php';
        
        $rg = new ReportGenerator();
        
        $fileName = 'conversionReport';
        
        $data = new xmlFlexy();    // data class
        $data->dateAlert = false;  // set the date alert to false, so no 'incomplete data' warning
        $data->maxVars   = 0;      // set the maximum # of tracker variables to 0
        
        // retrieve tracker info (Agency and Advertiser name)
        $trackerInfo = $this->_getTrackerInfo($trackerid);
        // set up the trackerInfo dates
        $trackerInfo['sd'] = $startdate;
	    $today = date('Y/m/d', strtotime("now"));
	    // set a dateAlert so that excel warns that data is incomplete
	    if ($enddate >= $today) {
	        $enddate = $today;
	        $data->dateAlert = $today;
	    }
        $trackerInfo['ed'] = $enddate;
        $data->trackerInfo = &$trackerInfo;
        
        // retrieve summary data
        $data->summary = $this->_getTrackerData($startdate, $enddate, $trackerid);
        
        // calculate the average basket value for each day
        $basketValues = $this->_getTrackerBasketValues($startdate, $enddate, $trackerid, $data);
        foreach($basketValues as $key=>$value) {
            $data->summary[$key]['avgBasketVal'] = $value['total_value'] / $value['num_values'];
        }

        // retrieve detailed data
        $data->details = $this->_getTrackerDetail($startdate, $enddate, $trackerid, $data);
        
        // extract the last row of 'details' so that the xls will be formated correctly
        $rg->formatFirstLastRow($data, 'details', false, $lastRow = 'lastDetails');
        
        // headers
        $rg->sendHeaders($fileName.'.xls');
        
        // compile
        $xml = $rg->compileXml($fileName.'.xml', $data, $options = null, "advertiser");
        
        echo $xml;
        flush();
        exit;
    }
    
    /**
    * Function to get information about the tracker
    *
    * @param  Integer $trackerid The tracker id
    * @return array Array of tracker related values.
    */
    function _getTrackerInfo($trackerid) {
        $conf = $GLOBALS['_MAX']['CONF'];
        
        $res = phpAds_dbQuery("
            SELECT
                ag.name AS agency_name,
                cl.clientname AS advertiser_name
            FROM
                {$conf['table']['prefix']}{$conf['table']['agency']} AS ag,
                {$conf['table']['prefix']}{$conf['table']['clients']} AS cl,
                {$conf['table']['prefix']}{$conf['table']['trackers']} AS tr
            WHERE
                cl.clientid = tr.clientid
            AND ag.agencyid = cl.agencyid
            AND tr.trackerid = {$trackerid}
        ");
        
        $row = phpAds_dbFetchArray($res);
        return $row;
    }
    
    /**
    * Function to count the tracker impressions (from split tables)
    *
    * @param  Date $startdate The start date/time to summarise from.
    * @param  Date $enddate The end date/time to summarise to.
    * @param  Integer $trackerid The tracker id
    * @return array $trackerData Associtive array of days => impressions.
    */
    function _getTrackerData($startdate, $enddate, $trackerid)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        
	    $start = date('Y-m-d', strtotime($startdate));
	    $end   = date('Y-m-d', strtotime($enddate));
	    $today = date('Y-m-d', strtotime("now"));
    
        $query = "
            SELECT
                s.day AS day,
                SUM(s.requests) AS requests,
                SUM(s.impressions) AS impressions,
                SUM(s.clicks) AS clicks,
                SUM(s.conversions) AS conversions
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_summary_ad_hourly']} AS s,
                {$conf['table']['prefix']}{$conf['table']['banners']} AS b,
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']} AS ct
            WHERE
                ct.trackerid={$trackerid}
            AND ct.campaignid=b.campaignid
            AND b.bannerid=s.ad_id
            AND s.day >= '$start'
            AND s.day <= '$end'
            GROUP BY
                day
            ORDER BY
                day
        ";
                
        $res = phpAds_dbQuery($query) or die("Error connecting to database");
        while ($row = phpAds_dbFetchArray($res)) {
            $trackerData[$row['day']] = $row;
            // check if date = today, and set dateAlert true
            if ($row['day'] == $today){
                $trackerData[$row['day']]['dateAlert'] = true;
            }
        }
        
        return $trackerData;
    }
    
    /**
     * Function to count the tracker basket values
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @param Integer $trackerid The tracker id
     * @param Object the data object
     * @return true
     */
    function _getTrackerBasketValues($startdate, $enddate, $trackerid, &$data) {
        $conf = $GLOBALS['_MAX']['CONF'];

	    $start = date('Y-m-d', strtotime($startdate));
	    $end   = date('Y-m-d', strtotime($enddate));
	    
        $res = phpAds_dbQuery("
            SELECT
                SUM(svv.value) as total_value,
                COUNT(svv.value) as num_values,
                DATE_FORMAT(con.tracker_date_time, '%Y-%m-%d') as day
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_connection']} AS con,
                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_variable_value']} AS svv,
                {$conf['table']['prefix']}{$conf['table']['variables']} AS v
            WHERE
                svv.data_intermediate_ad_connection_id=con.data_intermediate_ad_connection_id
            AND svv.tracker_variable_id=v.variableid
            AND v.is_basket_value=1
            AND con.tracker_id={$trackerid}
            AND con.tracker_date_time >= '".$start."'
            AND con.tracker_date_time <= '".$end."'
            GROUP BY
                day
            ");
        $totalBasketCount = 0;
        $totalBasketValue = 0;
        
        while ($row = phpAds_dbFetchArray($res)) {
            $data->summary[$row['day']]['avgBasketVal'] = $row['total_value'] / $row['num_values'];
            $totalBasketCount = $totalBasketCount + $row['num_values'];
            $totalBasketValue = $totalBasketValue + $row['total_value'];
        }
        
        $data->avgTotalBasketValue = $totalBasketValue / $totalBasketCount;

        return true;
    }
    
    /**
     * Function to retreive names of tracker variables
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @param Integer $trackerid The tracker id
     * @param Object the data object
     * @return true
     */    
    function _getTrackerVariableValues($tracker_id) {
        global $phpAds_config;
        
        $variableValuesRes = phpAds_dbQuery("
            SELECT
                variableid AS variable_id,
                trackerid AS tracker_id,
                name AS variable_name,
                description AS variable_description,
                variabletype AS variable_type,
                datatype AS variable_datatype,
                is_basket_value AS is_basket_value
            FROM
                {$phpAds_config['tbl_variables']} AS v
            WHERE
                v.trackerid = {$tracker_id}
        ");
        
        $variableValues = array();
        while ($variableValuesRow = phpAds_dbFetchArray($variableValuesRes)) {
            $variableValues[] = $variableValuesRow['variable_description'];
        }
        return $variableValues;
    }
    
    /**
    * Function to count the tracker impressions (from split tables)
    *
    * @param  Date $startdate The start date/time to summarise from.
    * @param  Date $enddate The end date/time to summarise to.
    * @param  Integer $trackerid The tracker id
    * @param  Object $data the data object
    * @return array $trackerDetails Associtive array of days => impressions.
    */
    function _getTrackerDetail($startdate, $enddate, $trackerid, &$data) {
        $conf = $GLOBALS['_MAX']['CONF'];
        
    	$dbStart = date('%Y-%m-%d %H:%M:%S', strtotime($startdate));
    	$dbEnd = date('%Y-%m-%d %H:%M:%S', strtotime($enddate));
        
        $trackerImpressionsRes = phpAds_dbQuery("
            SELECT
                con.tracker_date_time AS tracker_date_time,
                af.name AS affiliate_name,
                con.data_intermediate_ad_connection_id AS connection_id,
                con.connection_type AS connection_type,
                con.connection_date_time AS connection_date_time,
                b.description AS advertisment_name,
                z.zonename AS zone_name,
                con.connection_ip_address AS connection_ip_address,
                con.connection_country AS connection_country,
                con.tracker_domain AS tracker_domain,
                con.connection_domain AS connection_domain,
                con.connection_query AS connection_query,
                con.connection_language AS connection_language,
                con.connection_os AS connection_os,
                con.connection_browser AS connection_browser
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_connection']} AS con,
                {$conf['table']['prefix']}{$conf['table']['zones']} AS z,
                {$conf['table']['prefix']}{$conf['table']['affiliates']} AS af,
                {$conf['table']['prefix']}{$conf['table']['banners']} AS b
            WHERE
                con.tracker_id={$trackerid}
            AND con.tracker_date_time >= '".$dbStart."'
            AND con.tracker_date_time <= '".$dbEnd."'
            AND z.zoneid = con.zone_id
            AND z.affiliateid = af.affiliateid
            AND b.bannerid = con.ad_id
            ORDER BY
                tracker_date_time
        ");
                
        $connection_ids = array();
        while ($trackerImpressionRow = phpAds_dbFetchArray($trackerImpressionsRes)) {
            $trackerImpressionRow['connection_type'] = $trackerImpressionRow['connection_type'] == 0 ? 'view' : 'click';
            $trackerDetails[$trackerImpressionRow['connection_id']] = $trackerImpressionRow;
            $connection_ids[] = $trackerImpressionRow['connection_id'];
        }
        $connection_ids = implode(",", $connection_ids);
        
        if (!empty($connection_ids)) {
            $trackerVariableValuesRes = phpAds_dbQuery("
                SELECT
                    ivv.data_intermediate_ad_connection_id AS connection_id,
                    ivv.value AS variable_value,
                    v.description AS variable_name,
                    v.is_basket_value AS is_basket
                FROM
                    {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_variable_value']} as ivv,
                    {$conf['table']['prefix']}{$conf['table']['variables']} AS v
                WHERE
                    ivv.data_intermediate_ad_connection_id IN ({$connection_ids})
                    AND v.variableid=ivv.tracker_variable_id
            ");

            // use maxVars to determine the most Variables a tracker has, in order to display spreadsheet properly
            $maxVars = 0;
            while ($trackerVariableValuesRow = phpAds_dbFetchArray($trackerVariableValuesRes)) {
                $maxVarsTmp = count($trackerDetails[$trackerVariableValuesRow['connection_id']]['variables']);
                $trackerDetails[$trackerVariableValuesRow['connection_id']]['variables'][$maxVarsTmp][$trackerVariableValuesRow['variable_name']]['variable_value'] = $trackerVariableValuesRow['variable_value'];
                $trackerDetails[$trackerVariableValuesRow['connection_id']]['variables'][$maxVarsTmp][$trackerVariableValuesRow['variable_name']]['is_basket'] = $trackerVariableValuesRow['is_basket'];
                $data->trackerVariables[$trackerVariableValuesRow['variable_name']] = true;
                if ($maxVarsTmp > $maxVars){
                    $maxVars = $maxVarsTmp;
                }
                $maxVarsTmp=0;
            }
        }

        // set an array for maximum # of tracker variables
        for ($n=0; $n<=$maxVars; $n++) {
            $data->maxVarsArr[$n] = $n;
        }
        
        // set columnCount for excel (default is 19) (add 1 because maxVars is 0 based)
        $data->expandedColumnCount = 19 + $maxVars + 1;
        // set mergeAcross for excel (default is 15)
        $data->mergeAccross = 15 + $maxVars +1;

        // set the default values for variables
        foreach ($data->maxVarsArr as $num){
            foreach ($trackerDetails as $key=>$value) {
                if (!$trackerDetails[$key]['variables'][$num]) {
                    $trackerDetails[$key]['variables'][$num]['N/A']['variable_value'] = "N/A";
                }
            }
        }

        return $trackerDetails;
    }
}

?>
